@@ -57,7 +57,7 @@ public:
       GTEST_SKIP();
     TargetOptions Options;
     TM = std::unique_ptr<TargetMachine>(
-        T->createTargetMachine("X86", "", "", Options, std::nullopt));
+        T->createTargetMachine(TargetTriple, "", "", Options, std::nullopt));
     if (!TM)
       GTEST_SKIP();
     MMI = std::make_unique<MachineModuleInfo>(TM.get());
@@ -116,7 +116,7 @@ body:             |
     successors: %bb.2, %bb.4
     liveins: $rdi, $rsi
 
-    %1:gr32 = COPY $rsi
+    %1:gr64 = COPY $rsi
     %0:gr64 = COPY $rdi
     MOV64mr %1, 1, $noreg, 0, $noreg, %0 :: (store (s64) into %ir.p)
     %2:gr64 = SUB64ri32 %0, 1, implicit-def $eflags
@@ -206,7 +206,7 @@ body:             |
     successors: %bb.2, %bb.4
     liveins: $rdi, $rsi
 
-    %1:gr32 = COPY $rsi
+    %1:gr64 = COPY $rsi
     %0:gr64 = COPY $rdi
     MOV64mr %1, 1, $noreg, 0, $noreg, %0 :: (store (s64) into %ir.p)
     %2:gr64 = SUB64ri32 %0, 1, implicit-def $eflags
